from bs4 import BeautifulSoup

from kraken_extract_from_html.extractors.extract_feeds import extract_feeds
from kraken_extract_from_html.extractors.extract_images import extract_images
from kraken_extract_from_html.extractors.extract_links import extract_links
from kraken_extract_from_html.extractors.extract_schemas import extract_schemas
from kraken_extract_from_html.extractors.extract_src import extract_src
from kraken_extract_from_html.extractors.extract_text import extract_text
from kraken_extract_from_html.extractors.extract_title import extract_title

from kraken_extract_from_html.extractors.extract_title import extract_title
from kraken_extract_from_html.extractors.extract_tables import extract_tables
import requests
import datetime
import copy

from kraken_thing import Thing, Things

ASSET_ID = 'f83225cf-541c-379b-a6bd-b337d1139f6e'
LOG_URL = 'https://apidata.tactik8.repl.co/logs'
LOG_URL = 'https://data.krknapi.com/logs'


def get(url, html):
    return process_extraction(url, html)


def kraken_extract_from_url(url, contentUrl=None):

    contentUrl = contentUrl if contentUrl else url
    
    try:
        r = requests.get(contentUrl)
        html = r.text
    except Exception as e:
        print(e)
        return {}
    
    return process_extraction(url, html)


def kraken_extract_from_html(url, html):
    return process_extraction(url, html)

def kraken_extract_from_record(input_records):

    if not isinstance(input_records, list):
        input_records = [input_records]

    records = []

    for r in input_records:

        about = r.get('about', {})
        if isinstance(about, list) and len(about) > 0: 
            about = about[0]

        aboutUrl = r.get('url', None)
        if isinstance(aboutUrl, list) and len(aboutUrl) > 0: 
            aboutUrl = aboutUrl[0]
        htmls = r.get('text', None)
        archivedAt = r.get('archivedAt', None)

        if not htmls:
            htmls = []
            archivedAt = r.get('archivedAt', [])
            archivedAt = archivedAt if isinstance(archivedAt, list) else [archivedAt]
            for i in archivedAt:
                r = requests.get(i)
                html = r.text
                htmls.append(html)

        if not htmls:
            htmls = []
            urls = r.get('url', [])
            urls = urls if isinstance(urls, list) else [urls]
            for i in urls:
                r = requests.get(i)
                html = r.text
                htmls.append(html)


        for html in htmls:
            if aboutUrl and html:
                record = process_extraction(aboutUrl, html)
                records.append(record)


    return records



def process_extraction(url, html, image_urls = None):

    records = []

    
    text = extract_text(url, html)
    records+=text
    
    soup = _get_soup(html)


    links = extract_links(url, soup)
    records+=links


    images = extract_images(url, soup)
    records += images


    feeds = extract_feeds(url, soup)
    records += feeds

    try:
        schemas = extract_schemas(url, html)
        records += schemas
    except Exception as e:
        print('Error', e)
        a=1

    src = extract_src(url, soup)
    records += src

    texts = extract_text(url, html)
    records += texts

    titles = extract_title(url, soup)
    records += titles


    
    tables = extract_tables(url, html)
    records += tables

    action_record = get_action_record(url, records)

    log_action(action_record)
    
    return action_record




def _get_soup(html):

    soup = BeautifulSoup(html, 'html.parser')

    return soup



def get_action_record(url, records):
    ''' returns action record
    '''
    record = {
        '@type': 'Action',
        'name': 'api extract from html',
        'instrument': {
            "@id": ASSET_ID,
            "@type": "WebApplication"
        },
        'object': {
            '@type': 'WebPage',
            'url': url
        },
        'result': records,
        'actionStatus': 'completedActionStatus',
        'startTime': datetime.datetime.now().isoformat(),
        'endTime': datetime.datetime.now().isoformat()
    }

    return record


def log_action(input_record):
    """Log action to db
    """

    t = Thing()
    t.api.url = LOG_URL
    
    record = copy.deepcopy(input_record)
    record['result'] = len(record.get('result', []))

    t.load(record)
    t.api.post()
    return
                           
       



import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from unicodedata import normalize
import datetime
import uuid
from io import StringIO  


def get(url, html):
    
    return extract_tables(url, html)

def extract_tables(url, html):

    file = StringIO(html)
    try:
        tables = pd.read_html(file)
    except Exception as e:
        if str(e) != 'No tables found':
            print(e)
        return []
    
    
    records = []
    count = 0
    for t in tables:

        record = {
            '@type': 'table',
            '@id': str(uuid.uuid4()),
            'text': t.to_json(None,'records'),
            'dateCreated': datetime.datetime.now(),
            'url': url,
            'name': f'table_{str(count)}'
        }
        records.append(record)
        count += 1

    return records




from boilerpy3 import extractors
import re


def get(url, html):
    return extract_text(url, html)



def extract_text(url, html):

    # Get text from webpage



    extractor = extractors.ArticleExtractor()

    text = extractor.get_content(html)

    # Condenses all repeating newline characters into one single newline character

    text = '\n'.join([p for p in re.split('\n|\r', text) if len(p) > 0])


    record = [{
        '@type': 'schema:webpage',
        'schema:url': url,
        'schema:text': text
    }]



    return record





import urllib.parse

def get(url, soup):
    return extract_links(url, soup)


def extract_links(url, soup):

    links = []

    # Get links
    href_links = soup.find_all('a')
    
    for link in href_links:
        link_url = link.get('href', None)
        link_title = link.text

        # Clean link title
        link_title = link_title.replace('\n', '')
        link_title = link_title.replace('\r', '')
        link_title = link_title.replace('  ', ' ')
        link_title = link_title.strip()


        # Convert to absolute url
        link_url = urllib.parse.urljoin(url, link_url)

        # Compile into structured record
        link_record = {
            '@type': 'schema:webpage',
            'schema:title': link_title,
            'schema:url': link_url
        }

        # Add to list if url exist
        if link_record.get('schema:url', None):
            links.append(link_record)

    return links



from kraken_extract_from_html import kraken_extract_from_html as k
import os
from flask import Flask
from flask import request
from flask import Response
from flask import redirect
from flask import url_for
from flask import jsonify

from flask_cors import CORS

import datetime
import time
import uuid
import random
import json
import markdown
import requests
from kraken_thing import Thing, Things
ASSET_ID = 'f83225cf-541c-379b-a6bd-b337d1139f6e'
LOG_URL = 'https://apidata.tactik8.repl.co/logs'
LOG_URL = 'https://data.krknapi.com/logs'


# Initalize app
test_mode = False


# Initialize flask app
app = Flask(__name__,
            static_url_path='',
            static_folder='static',
            template_folder='templates')
app.secret_key = b'_5#mn"F4Q8z\n\xec]/'
app.config['SEND_FILE_MAX_AGE_DEFAULT'] = 0
CORS(app)
LOG_URL = 'https://data.krknapi.com/logs'

    
@app.route('/', methods=['GET'])
def main_get():
    """Process get data
    """

    # Accept and process url parameter
    url = request.args.get('url')
    contentUrl = request.args.get('contentUrl')

    
    if url:
        return jsonify(k.kraken_extract_from_url(url, contentUrl))
        

    # Shows instructions
    with open('README.md') as f:
        md_content = f.read()

    html_content = markdown.markdown(md_content)

    return Response(html_content)


@app.route('/', methods=['POST'])
def main_post():
    """Process post data
    """

    input_records = request.get_json()

    result = k.kraken_extract_from_record(input_records)
    
    return jsonify(result)

@app.route('/log', methods=['GET'])
def log_get():
    """
    """
    url = LOG_URL

    t = Thing()
    t.api_url(LOG_URL)
    
    things = Things()

    things.api_get({})

    content = things.html.table()
    
    return Response(content)
    


def run_api():
    #from waitress import serve
    #serve(app, host="0.0.0.0", port=8080, threads= 20)
    app.run(host='0.0.0.0', debug=False)








def get(emails):
    return _extract_emails(emails)

def extract_emails(emails):
    return _extract_emails(emails)



def _extract_emails(emails):

    schemas = []
    for email in emails:
        record = {
            '@type': 'schema:contactPoint',
            'schema:email': email
            }

        schemas.append(record)

    return schemas



import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="kraken-extract-from-html",
    version="0.0.20",
    description="Kraken Extract From HTML",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/tactik8/kraken_extract_from_html2",
    author="Tactik8",
    author_email="info@tactik8.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=['beautifulsoup4', 'extruct', 'html2text',  'ioc-finder','w3lib', 'boilerpy3', 'markdown', 'pandas', 'matplotlib', 'kraken-thing'],
    
)






import urllib.parse




def get(url, soup):

    return extract_images(url, soup)




def extract_images(url, soup):

    images = []
    ref_images = soup.findAll('img')

    for img in ref_images:

        image_url = img.get('src', '')


        new_i = image_url
        if new_i.startswith('//'):
            new_i = 'https:' + new_i

        # Convert to absolute url
        image_url = urllib.parse.urljoin(url, new_i)

        # Compile into structured record
        image = {
            '@type': 'schema:ImageObject',
            'schema:title': img.get('title', None),
            'schema:contentUrl': new_i,
            'schema:url': url
        }
        
        # Skip if already in list
        if image in images:
            continue



        # POst thumbnail on image sharing site
            

        # Add to list if url exist
        if image.get('schema:contenturl', None):
            images.append(image)
    
    return images

